----------------------------------------------------------------------------------

-- This code is optimized for clock frequency of 1 Mhz.
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity lcd is
    Port ( DB : out  STD_LOGIC_VECTOR (7 downto 0);
           RS : out  STD_LOGIC;
           RW : out  STD_LOGIC;
           EN : out  STD_LOGIC;
           CLK : in  STD_LOGIC;
			  RESET : in  STD_LOGIC);  -- active low
end lcd;

architecture Behavioral of lcd is
type lcd_machine is (Power_Up_Delay,
							 Function_Set,
							 Display_Control,
							 Clear_Display,
							 Return_Home,
							 Char_Display,
							 Shift_Left1,
							 Delay1,
							 Hold,
							 Shift_Left2,
							 Delay2);
signal lcd_state : lcd_machine:= Power_Up_Delay;	
attribute fsm_encoding: string;
attribute fsm_encoding of lcd_state : signal is "compact"; -- This tells XST to use binary encoding

signal delay_count : std_logic_vector(20 downto 0):= "0"&X"00000";
signal lcd_data : std_logic_vector(9 downto 0):= "00"&X"00";
signal lcd_enable : std_logic:= '0';
signal shift_count : std_logic_vector(3 downto 0):= X"0";
signal wait_count : std_logic_vector(23 downto 0):= X"000000";
type LCD_Cmd_Table is array(integer range <>) of std_logic_vector(9 downto 0);
constant LCD_Cmd : LCD_Cmd_Table:= (0 => "00"&X"00",
												1 => "00"&X"3C", -- Function Set: Data length to be 8 bits (DL)
																	  -- 2 line mode (N), 5x11 dot format	(F)
												2 => "00"&X"0C", -- Display Control : Enable both lines (D),
																	  -- Cursor OFF (C), Blink OFF (B)
												3 => "00"&X"01", -- Clear Display : DDRAM cleared, address = 00
												4 => "00"&X"02", -- Return Home : DDRAM address=00
												5 => "00"&X"94", -- set DDRAM pointer for 1st line 
																	  -- offset to center word 'Xilinx' on top line
											   6 => "10"&X"58", -- X
											   7 => "10"&X"69", -- i
											   8 => "10"&X"6C", -- l
											   9 => "10"&X"69", -- i
											   10 =>"10"&X"6E", -- n
											   11 =>"10"&X"78", -- x
											   12 => "00"&X"d1", -- set DDRAM pointer for 2nd line
											   13 => "10"&X"43", -- C
											   14 => "10"&X"6f", -- o
											   15 => "10"&X"6f", -- o
											   16 => "10"&X"6C", --l
											   17 => "10"&X"72", --r
											   18 => "10"&X"75", --u
											   19 => "10"&X"6E", --n
											   20 => "10"&X"6E", --n
											   21 => "10"&X"65", --e
											   22 => "10"&X"72", --r
											   23 => "10"&X"B0", -- -
											   24 => "10"&X"49", -- I
											   25 => "10"&X"49"  -- I
											   );
											  

signal LCD_Ptr : integer range 0 to LCD_Cmd'HIGH:= 0;
begin
process(CLK, RESET)
begin
if (RESET='0') then
   lcd_state <= Function_Set;
	delay_count <= (others=>'0');
	LCD_Ptr <= 1;
	shift_count <= (others=>'0');
	wait_count <= (others=>'0');
elsif (CLK = '1' and CLK'event) then
	case lcd_state is
	when Power_Up_Delay =>
		if (delay_count = "0"&X"07D00") then   --- wait 32 milli-seconds
			delay_count <= "0"&X"00000";
			lcd_state <= Function_Set;
			LCD_Ptr <= LCD_Ptr + 1;
		else
			lcd_data <= LCD_Cmd(LCD_Ptr);
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Power_Up_Delay;
		end if;
	when Function_Set =>
		if (delay_count = "0"&X"00028") then				-- 40 micro-seconds
			delay_count <= "0"&X"00000";
			lcd_state <= Display_Control;
			LCD_Ptr <= LCD_Ptr + 1;
		elsif (delay_count = "0"&X"00002") then			-- 1 uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Function_Set;
		elsif (delay_count = "0"&X"00001") then			-- 1 uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Function_Set;
		else	
			lcd_data <= LCD_Cmd(LCD_Ptr);					
			delay_count <= delay_count + 1;
			lcd_state <= Function_Set;
		end if;
	when Display_Control =>
		if (delay_count = "0"&X"00028") then				-- 40 micro-seconds
			delay_count <= "0"&X"00000";
			lcd_state <= Clear_Display;
			LCD_Ptr <= LCD_Ptr + 1;
		elsif (delay_count = "0"&X"00002") then			-- 1 uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Display_Control;
		elsif (delay_count = "0"&X"00001") then			-- 1uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Display_Control;
		else	
			lcd_data <= LCD_Cmd(LCD_Ptr);
			delay_count <= delay_count + 1;
			lcd_state <= Display_Control;
		end if;
	when Clear_Display =>
		if (delay_count = "0"&X"00668") then				-- 1.64 milli-seconds
			delay_count <= "0"&X"00000";
			lcd_state <= Return_Home;
			LCD_Ptr <= LCD_Ptr + 1;
		elsif (delay_count = "0"&X"00002") then			-- 1 uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Clear_Display;
		elsif (delay_count = "0"&X"00001") then			-- 1 uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Clear_Display;
		else	
			lcd_data <= LCD_Cmd(LCD_Ptr);
			delay_count <= delay_count + 1;
			lcd_state <= Clear_Display;
		end if;
	when Return_Home =>
		if (delay_count = "0"&X"00668") then				-- 1.64 milli-seconds
			delay_count <= "0"&X"00000";
			if (LCD_Ptr = LCD_Cmd'HIGH ) then --25
				lcd_state <= Shift_Left1;
			else
				lcd_state <= Char_Display;
				LCD_Ptr <= LCD_Ptr + 1;
			end if;
		elsif (delay_count = "0"&X"00002") then			-- 1uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Return_Home;
		elsif (delay_count = "0"&X"00001") then			-- 1uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Return_Home;
		else	
			lcd_data <= LCD_Cmd(4);			                    
			delay_count <= delay_count + 1;
			lcd_state <= Return_Home;
		end if;
	when Char_Display =>
		if (delay_count = "0"&X"00028") then		-- 40 micro-seconds
			delay_count <= "0"&X"00000";
			if (LCD_Ptr = LCD_Cmd'HIGH ) then		-- shift in the array to the LCD
				lcd_state <= Shift_Left1;
			else
				lcd_state <= Char_Display;
				LCD_Ptr <= LCD_Ptr + 1;
			end if;
		elsif (delay_count = "0"&X"00002") then			-- 1uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Char_Display;
		elsif (delay_count = "0"&X"00001") then			-- 1uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Char_Display;
		else	
			lcd_data <= LCD_Cmd(LCD_Ptr);
			delay_count <= delay_count + 1;
			lcd_state <= Char_Display;
		end if;
	when Shift_Left1 =>
		if (delay_count = "0"&X"00028") then				-- 40 micro-seconds
			delay_count <= "0"&X"00000";
			shift_count <= shift_count + 1;
			lcd_state <= Delay1;
		elsif (delay_count = "0"&X"00002") then			-- 1uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left1;
		elsif (delay_count = "0"&X"00001") then			-- 1uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left1;
		else	
			lcd_data <= "00"&X"18";
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left1;
		end if;
	when Delay1 =>
		if (wait_count = X"00c350") then   -- scroll in 1 char every 50 mS
			wait_count <= X"000000";
			if (shift_count = X"F") then   -- execute 15 shifts
				shift_count <= X"0";
				lcd_state <= Hold;
			else
				lcd_state <= Shift_Left1;
			end if;
		else
			lcd_state <= Delay1;
			wait_count <= wait_count + 1;
		end if;
	when Hold =>
		if (wait_count = X"02dc6c0") then  -- 3 seconds hold the text
			wait_count <= (others=>'0');
			lcd_state <= Shift_Left2;
		else
			wait_count <= wait_count + 1;
			lcd_state <= Hold;
		end if;
	when Shift_Left2 =>
		if (delay_count = "0"&X"00028") then				-- 40 micro-seconds
			delay_count <= "0"&X"00000";
			shift_count <= shift_count + 1;
			lcd_state <= Delay2;
		elsif (delay_count = "0"&X"00002") then			-- 1uS 
			lcd_enable <= '0';
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left2;
		elsif (delay_count = "0"&X"00001") then			-- 1uS 
			lcd_enable <= '1';
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left2;
		else	
			lcd_data <= "00"&X"18";
			delay_count <= delay_count + 1;
			lcd_state <= Shift_Left2;
		end if;
	when Delay2 =>
		if (wait_count = X"00c350") then  -- scroll off 1 char every 50 mS
			wait_count <= X"000000";
			if (shift_count = X"F") then  -- perform 15 shifts
				shift_count <= X"0";
				lcd_state <= Return_Home;
			else
				lcd_state <= Shift_Left2;
			end if;
		else
			wait_count <= wait_count + 1;
			lcd_state <= Delay2;
		end if;
	end case;
end if;
end process;

DB <= lcd_data(7 downto 0);
RS <= lcd_data(9);
RW <= lcd_data(8);
EN <= lcd_enable;

end Behavioral;