----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    14:50:39 03/04/2008 
-- Design Name: 
-- Module Name:    top - Behavioral 
-- Project Name:   DemoMain
-- Target Devices: CR-II 256 TQ144
-- Tool versions: ISE 10.1i 
-- Description:  Design targeted for the CoolRunner-II Starter Board with PMod CLP module
-- 					Implements a counter on the quad 7-segment display, that is reset by
--						BTN0. BTN1 is a clock enable for the counter.
--						SW0 is the Data Gate control, which disables the clock.
--						When the PMod CLP is attached to J3 and J4, applying the reset will
--						initialize the display, then scroll the text 'Xilinx CoolRunner-II'.
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library UNISIM;
use UNISIM.VComponents.all; -- this library is necessary because the CRII clock 
									 -- divider component is used

entity top is
    Port ( clk : in  STD_LOGIC; -- main clock (intended for 1 Mhz)
           sw0 : in  STD_LOGIC; -- data_gate enable
           rst : in  STD_LOGIC; -- active low, btn0
           cken : in  STD_LOGIC; -- active low, btn1
           sseg : out  STD_LOGIC_VECTOR (07 downto 0); -- output to quad 7-segment display
           sseg_sel : out  STD_LOGIC_VECTOR (03 downto 0); -- selector for the 7-segment display
           leds : out  STD_LOGIC_VECTOR (03 downto 0); -- LED outputs 
			  db : out STD_LOGIC_VECTOR (7 downto 0); -- following 5 lines are for LCD PMOD
			  rs : out STD_LOGIC;
			  rw : out STD_LOGIC;
			  en : out  STD_LOGIC );
end top;

architecture Behavioral of top is

component lcd Port (DB : out  STD_LOGIC_VECTOR (7 downto 0);
           RS : out  STD_LOGIC;
           RW : out  STD_LOGIC;
           EN : out  STD_LOGIC;
           CLK : in  STD_LOGIC;
			  RESET : in  STD_LOGIC);
end component;

component hex2led
    Port ( HEX : in  STD_LOGIC_VECTOR (3 downto 0);
				LED :out STD_LOGIC_VECTOR (6 downto 0 ));
end component;

component ten_counter
    Port (clk : in std_logic;
			 rst : in std_logic;
			 ce : in std_logic;
			 dout : out std_logic_vector (3 downto 0);
			 co : out std_logic);
end component;

component sixty_counter
	 Port (clk : in std_logic;
	       ce : in std_logic;
			 rst : in std_logic;
			 dout : out std_logic_vector (3 downto 0);
			 co : out std_logic);
end component;


component one_hundred_k_counter
    Port (clk : in std_logic;
		    cken : in std_logic;
			 rst : in std_logic;
			 dout_fast : out std_logic;
			 dout_100k : out std_logic);
end component;

signal clkdv : std_logic ; -- output of clock divider
signal lcd_A, lcd_B, lcd_C, lcd_D : std_logic_vector (6 downto 0); -- LCD data
signal seconds, tens, minute_lsb, minute_msb : std_logic_vector (3 downto 0);
signal sec_co, ten_co, sixty_co, minute_lsb_co : std_logic; -- carry out for counter digits
signal lcd_cycle_reg : std_logic_vector (3 downto 0) := "0111"; -- used to cycle the select line for the LED digits
signal second_clk : std_logic;
signal minute_enable, ten_minute_enable: std_logic;
signal fast_clk : std_logic; -- a fast clock to cycle the LED select line 

begin

cr_clk_divider : clk_div10 port map (
      CLKDV => CLKDV,    -- Divide 1 Mhz to 100 kHz
      CLKIN => CLK 
);

lcd_driver : lcd port map (
			  DB => db,
           RS => rs,
           RW => rw,
           EN => en,
           CLK => clk,
			  RESET => rst);

-- ledX: to convert values to the 7-segment display format

led1 : hex2led port map (
 hex => seconds,
 Led => lcd_A );
 
led2 : hex2led port map (
 hex => tens,
 Led => lcd_B );

led3 : hex2led port map (
 hex => minute_lsb,
 Led => lcd_C );

led4 : hex2led port map (
 hex => minute_msb,
 Led => lcd_D );

-- my_100k_counter: Divided 100 kHz output of clock divder to 1 Hz

my_100k_counter : one_hundred_k_counter port map (
  clk => clkdv,
  cken => cken,
  rst => rst,
  dout_fast => fast_clk,
  dout_100k => second_clk);

seconds_counter : ten_counter port map (
 clk => second_clk,
 ce => '1',
 rst => rst,
 dout => seconds,
 co => sec_co);

tens_counter : sixty_counter port map (
 clk => second_clk,
 ce => sec_co,
 rst => rst,
 dout => tens,
 co => ten_co);

minute_enable <= ten_co and sec_co;

ten_minute_enable <= minute_lsb_co and ten_co and sec_co;

min_counter_lsb : ten_counter port map (
 clk => second_clk,
 ce => minute_enable,
 rst => rst,
 dout => minute_lsb,
 co => minute_lsb_co );
 
min_counter_msb : ten_counter port map (
 clk => second_clk,
 ce => ten_minute_enable,
 rst => rst,
 dout => minute_msb );

-- cycle between the 4 7-segment displays at a fast rate, so they appear enabled simultaneously

lcd_cycle : process (fast_clk, rst, lcd_cycle_reg)
begin

  if (rst='0') then
     lcd_cycle_reg <= "1110";
	  elsif (fast_clk'event and fast_clk='1') then
	     lcd_cycle_reg (2 downto 0) <= lcd_cycle_reg (3 downto 1);
		  lcd_cycle_reg (3) <= lcd_cycle_reg(0);
		  end if;
 
end process;
 
sseg_sel <= lcd_cycle_reg;

-- mux for the 4 outputs to the 7 segment display

process (lcd_cycle_reg,lcd_A, lcd_B, lcd_C, lcd_D)
begin
   case lcd_cycle_reg is
      when "0111" => sseg <= ('1'& lcd_A);
      when "1011" => sseg <= ('1'& lcd_B);
      when "1101" => sseg <= ('0'& lcd_C);
      when "1110" => sseg <= ('1'& lcd_D);
      when others => sseg <= "11111111";
   end case;
end process;

leds(3) <= not (second_clk); -- second 'heartbeat' 
leds(2) <= '1';  -- disable this LED
leds(1) <= '1';  -- disable this LED
leds(0) <= not (sw0); -- led is active low, will light if data_Gate enabled

end Behavioral;

